@@ -5,8 +5,10 @@ from feeluown.library import AbstractProvider, ProviderV2, ProviderFlags as PF,
     NoUserLoggedIn, LyricModel, ModelNotFound
 from feeluown.media import Quality, Media
 from feeluown.library import ModelType, SearchType
+from feeluown.utils.dispatch import Signal
 from feeluown.utils.reader import create_reader, SequentialReader
 from .api import API, CodeShouldBe200
+from .login_controller import LoginController
 
 
 logger = logging.getLogger(__name__)
@@ -33,6 +35,7 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
     def __init__(self):
         super().__init__()
         self.api = API()
+        self.current_user_changed = Signal()
 
     @property
     def identifier(self):
@@ -42,11 +45,21 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
     def name(self):
         return '网易云音乐'
 
+    def auto_login(self):
+        logger.info("Try auto logging...")
+        user = LoginController.load()
+        if user is not None:
+            self.auth(user)
+            logger.info(f"Auto logging ok, user: {user.name}")
+        else:
+            logger.info("Auto logging failed, no user found.")
+
     def auth(self, user):
         cookies, exists = user.cache_get('cookies')
         assert exists and cookies is not None
         self._user = user
         self.api.load_cookies(cookies)
+        self.current_user_changed.emit(user)
 
     def has_current_user(self):
         return self._user is not None
@@ -54,7 +67,14 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
     def get_current_user(self):
         if self._user is None:
             raise NoUserLoggedIn
+        # 理论上，self._user 的 cookies 缓存总是存在的。
+        # 因为在创建 user 的各个逻辑里面，cookies 字段是带上的。
+        if isinstance(self._user, UserModel):
+            return self._user
         user = self.user_get(self._user.identifier)
+        cookies, exists = self._user.cache_get('cookies')
+        if exists:
+            user.cache_set('cookies', cookies)
         return user
 
     def user_get(self, identifier):
@@ -471,6 +491,23 @@ class NeteaseProvider(AbstractProvider, ProviderV2):
         result = _deserialize(data, NeteaseSearchSchema)
         return result
 
+    def get_user_from_cookies(self, cookies):
+        """Get a user model with cookies
+        """
+        if not cookies:  # is None or empty
+            raise ValueError('empty cookies')
+
+        provider.api.load_cookies(cookies)
+        try:
+            data = provider.api.user_level()
+        except CodeShouldBe200 as e:
+            raise ValueError(f'get user info with cookies failed: {e}')
+        else:
+            user_id = data['userId']
+            user = self.user_get(user_id)
+            user.cache_set('cookies', cookies)
+            return user
+
 
 provider = NeteaseProvider()
 
